%   Performance of error correction codes
%   
%   This code plots the BER and FER of Turbo codes
%   that employ the same block length n and arbitrary rates r.
% 
%   version 1.0 - September, 2016
%   
%   Authors: Rodrigo C. de Lamare

addpath(genpath('./Turbo Codes')); %All helping routines.

% Parameters:
n = 256; % block length
r = 0.5; % rate
k = 128; % message length
snr_db = 0:2:6; % range of snr values in dB assuming snr = Eb/N0 
n_blocks = 1000; % number of transmitted blocks for Montecarlo simulations
ber_turbo  = zeros(1,length(snr_db)); % vector to store ber values for turbo codes
fer_turbo  = zeros(1,length(snr_db)); % vector to store fer values for turbo codes
e_min = 5;% The minimum number of errors for statistical significance
  
% Parameters for turbo codes
% Code generator
g = [ 1 1 1; 1 0 1];
[ns,ks] = size(g); 
ms = ks - 1;
nstates = 2^ms;
puncture = 1;
r_t = 1/(2+puncture);   %   Code rate
I_turbo = 5;   %    Turbo iterations

% Block Error Record 
be_record = zeros(1,length(snr_db)); 


for j=1:length(snr_db)
    tt=tic();
    Lc_turbo = 4*r_t*10^(snr_db(j)/10); % reliability value for turbo codes
    sigma_turbo = 1/sqrt(2*r_t*10^(snr_db(j)/10)); 
    for l=1:n_blocks
        tic
        m = (rand(k,1)>0.5); % message vector
        % Turbo encoding
        [temp, alpha] = sort(rand(k,1));        % random interleaver mapping
        c_turbo = encoderm( m(1:k-ms).', g, alpha, puncture); % encoder output (+1/-1)
        % Received data vector
        r_turbo = c_turbo.' + sigma_turbo*randn(k*(2+puncture),1); % received bits
        r_turbo_demux = (Lc_turbo/2)*demultiplex(r_turbo,alpha,puncture); % demultiplex and scale to get input for decoder 1 and 2
        % Decoding
        m_est_turbo = turbo_iter(k,g,r_turbo_demux,alpha,I_turbo); % Turbo decoding
        % Error counting
        nfails_turbo = sum(m_est_turbo(1:end-ms).' ~= m(1:k-ms));        
        fer_turbo(j) = fer_turbo(j) + (nfails_turbo > 0);
        ber_turbo(j) = ber_turbo(j) + nfails_turbo;
        % Stopping criterion based on error
        error = nfails_turbo;
        be_record(j)= error;
        if (error >= e_min)
            fprintf(' Required number of errors obtained\n');
            %pause;
            break;
        end
    end
    ber_turbo(j) = ber_turbo(j)/(k*l); 
    fer_turbo(j) = fer_turbo(j)/l;
    toc(tt);
end

 subplot(121);
 semilogy(snr_db,fer_turbo,'m-*'); grid on;
 axis([0 6 0.00001 1]);
 xlabel('E_b/N_0 (dB)');
 ylabel('FER');
 legend('Turbo');

 subplot(122);
 semilogy(snr_db,ber_turbo,'m-*'); grid on;
 axis([0 6 0.00001 1]);
 xlabel('E_b/N_0 (dB)');
 ylabel('BER');
 legend('Turbo');



